### Multinomial Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("reshapedata2009.csv",header=TRUE)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
#important
b0 <- numeric(9)
#b0 <- c(1,1,1,1,1)

## Recall that we estimate parameters by maximizing the log-likelihood function, hence we first need to define that function

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  ## Alternative specific constants
  #constant value
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]
  
  #Toyosu Dummy
  c1 <- x[4]
  
  #day 
  d1 <- x[5]
  d2 <- x[6]
  d3 <- x[7]
  d4 <- x[8]
  
  #pre-Fri dummy
  e1 <- x[9]
  
  

  
 
  ## Travel time to destination
  #  d1 <- x[5]
  
  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes???
  ## ??????(return)
  ## ??????(bus)
  ## ?????????(recreation)
  ## ?????????(shopping)
  ## ??????(restaurant)
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
  # time           # fare      # constant
  return  <- exp(b1*matrix(1,nrow =hh,ncol=1)+c1*Data$Toyosu+d1*Data$Tuesday+d2*Data$Wednesday+d3*Data$Thursday+d4*Data$Friday+e1*Data$Toyosu*Data$Friday)
  recreation    <- exp(b2*matrix(1,nrow =hh,ncol=1))
  shopping   <- exp(b3*matrix(1,nrow =hh,ncol=1))
  restaurant   <- exp(1)
  
  ### Calculate the choice probabilities
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (recreation + return + shopping + restaurant)
  
  ## Calculate indiviudal choice probabilities
  Preturn <- (return / deno)
  Precreation   <- (recreation   / deno)
  Pshopping  <- (shopping  / deno)
  Prestaurant  <- (restaurant  / deno)
  
  ## Avoid problems stemming from choice probabilities becoming zero.???????????????0?????????????????????????????????????????????????????????
  Preturn <- (Preturn!=0)*Preturn + (Preturn==0)
  Precreation   <- (Precreation  !=0)*Precreation   + (Precreation  ==0)
  Pshopping  <- (Pshopping  !=0)*Pshopping + (Pshopping  ==0)
  Prestaurant  <- (Prestaurant!=0)*Prestaurant   + (Prestaurant  ==0)
  
  
  
  ## Choice results
  Creturn   <- Data$purpose =="return"
  Crecreation     <- Data$purpose =="recreation"
  Cshopping    <- Data$purpose =="shopping"
  Crestaurant    <- Data$purpose =="restaurant"
  
  ## Calculate the Log-likelihood function
  LL <- colSums(Creturn*log(Preturn) +
                  Crecreation  *log(Precreation)   + Cshopping  *log(Pshopping) +Crestaurant *log(Prestaurant))
  
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr,gr=NULL, method = "BFGS", hessian = TRUE, control=list(fnscale=-1))

## Parmeter estimation???Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)
