########## Nested Logit Model #########

#<<<First step>>>
##### Data Reading from csv file#####
  data <- read.csv("transport.csv")
  names(data)
  summary(data)

#<<<Second step>>>
#####Defining Log-likehood function#####
LL <- function(x) 
{
    Para_time <- x[1]
    Para_cost <- x[2]
    Ctrain    <- x[3]
    Ctaxi     <- x[4]
    LogsumRT  <- x[5]

  # Utility function
    Vcar   <- data$Carcost*Para_cost + data$Cartime*Para_time
    Vtrain <- data$Skrfare*Para_cost + data$Skrtime*Para_time + Ctrain
    Vtaxi  <- data$Taxfare*Para_cost + data$Taxtime*Para_time + Ctaxi
    VRT    <- 2
    
  # Log-sum variable
    LS <- log(exp(Vcar/LogsumRT)+exp(Vtaxi/LogsumRT))
  
  # Denominator
    Sum  <- exp(Vcar/LogsumRT) + exp(Vtaxi/LogsumRT)
    Sum2 <- exp(Vtrain)+exp(VRT+LogsumRT*LS)
  
  # Probabilites
    PcarRT  <- exp(Vcar/LogsumRT)/Sum
    PtaxiRT <- exp(Vtaxi/LogsumRT)/Sum
    PRT     <- exp(VRT + LogsumRT*LS)/Sum2
    Ptrain  <- exp(Vtrain)/Sum2
    Pcar    <- PcarRT*PRT
    Ptaxi   <- PtaxiRT*PRT
    
  # Dummies for choice
    Dcar   <- ifelse(data$choice == 2,1,0)
    Dtrain <- ifelse(data$choice == 1,1,0)
    Dtax   <- ifelse(data$choice == 3,1,0)
    DRT    <- ifelse(data$choice > 1,1,0)
  
  # Log-likelihood function
    L <- sum(Dcar*log(Pcar) + Dtrain*log(Ptrain) + Dtax*log(Ptaxi) + DRT*log(PRT))
    return(L)
  }

  #<<<Third step>>>
  #####Model Estimation#####
    initValue <- c(0,0,0,0,1)
    NL <- optim(initValue, LL, method="BFGS",hessian=TRUE, 
                 control=list(maxit=10000,fnscale=-1))
  
  #<<<Fourth step>>>
  #####Confirming estimation results#####
  # Estimated Parameters
    parameter <- NL$par
  
  # Hessian (for t-values)
    hhh <- NL$hessian
  
  # t-values
    t_stat <- parameter/sqrt(-diag(solve(hhh)))
    
  # initial log-likelihood
    LL0 <- NROW(data)*log(1/3)
    
  # converged log-likelihood
    LL1 <- NL$value
    
  # MacFadden's Rho square
    rho <- 1-(LL1/LL0)
    rho.adj <- 1-((LL1-length(parameter))/LL0)
    
  # Display all estimation results
    results <- cbind(parameter,t_stat)
    rownames(results) <- c("Para_cost","Para_time","Ctrain","Ctaxi","LogsumRT")
    GoF <- cbind(LL0, LL1, rho, rho.adj)    # Goodness of fit
    rownames(GoF) <- c("GoF")
  
    print(results)
    print(GoF)




